import { MigrationInterface, QueryRunner, Table, TableForeignKey } from "typeorm";
import { DB_TYPE } from "../env.defaults";

export class Initial1724317398939 implements MigrationInterface {
  name = "Initial1724317398939";

  public async up(queryRunner: QueryRunner): Promise<void> {
    const variableType_int = DB_TYPE == "mysql" ? "int" : "integer";

    await queryRunner.createTable(
      new Table({
        name: "user",
        columns: [
          { name: "id", type: variableType_int, isPrimary: true, isGenerated: true, generationStrategy: "increment" },
          { name: "mail", type: "varchar", length: "255", isNullable: false },
          { name: "username", type: "varchar", length: "255", isNullable: false },
          { name: "secret", type: "varchar", length: "255", isNullable: false },
        ],
      }),
      true
    );

    await queryRunner.createTable(
      new Table({
        name: "refresh",
        columns: [
          { name: "id", type: variableType_int, isPrimary: true, isGenerated: true, generationStrategy: "increment" },
          { name: "token", type: "varchar", length: "255", isNullable: false },
          { name: "expiry", type: "datetime", isNullable: false },
          { name: "userId", type: variableType_int, isNullable: false },
        ],
      }),
      true
    );

    await queryRunner.createForeignKey(
      "refresh",
      new TableForeignKey({
        columnNames: ["userId"],
        referencedColumnNames: ["id"],
        referencedTableName: "user",
        onDelete: "CASCADE",
        onUpdate: "RESTRICT",
      })
    );
  }

  public async down(queryRunner: QueryRunner): Promise<void> {
    const table = await queryRunner.getTable("refresh");
    const foreignKey = table.foreignKeys.find((fk) => fk.columnNames.indexOf("userId") !== -1);
    await queryRunner.dropForeignKey("refresh", foreignKey);
    await queryRunner.dropTable("refresh");
    await queryRunner.dropTable("user");
  }
}
